Program Macro;
{$E+,N+,G+,B-}


Type
  Float   = Extended;
Const
  eps     = 1e-15;
Var
  N, M          : Integer;
  L             : LongInt;
  T             : Float;
  Gr            : Array [1..32, 1..32] Of Boolean;
  G, W          : Array [1..32, 1..32] Of Float;
  R             : Array [1..32] Of Boolean;
  V             : Array [1..32] Of LongInt;
  Await         : Array [1..32] Of Float;
  Tm, Nt        : Float;
  Lst           : Integer;




Procedure MakeInput;
Var i, j, k : Integer;
Begin
  Assign (Input, 'INPUT.TXT');
  Reset (Input);
  Read (N, M, L, T);
  For i:=1 To N Do
    For j:=1 To N Do
      Begin Gr[i,j]:=False; G[i,j]:=1e20 End;
  For k:=1 To M Do
    Begin
      Read (i, j);
      Gr [i,j]:= True; Gr[j,i]:=True;
      Read (G[i,j]);
      G[j,i] := G[i,j]
    End
End;

Procedure InitMonsters;
Var i, j  : Integer;
Begin
  Tm := 0;
  Nt := 0;
  Lst := 1;
  For i:=1 To N Do R[i]:=False;
  R[1] := True;
  For i:=1 To N Do V[i] := L;
  V[1] := 0;
  For i:=1 To N Do Await[i] := 0;
  For i:=1 To N Do
    For j:=1 To N Do
      W[i,j]:=0
End;

Function All : Boolean;
Var i : Integer;
Begin
  For i:=1 To N Do
    If (V[i]=0) And (Await[i]<eps) And (Not R[i]) Then
      Begin R[i]:=True; Lst:=i End;
  All := False;
  For i:=1 To N Do
    If Not R[i] Then Exit;
  All := True
End;

Function GetRTime (i : Integer) : Float;
Var
  S0, S1, Ti, Tt: Float;
  j, k          : Integer;
  CW            : Array [1..32] Of Float;
  RT            : Float;
  RD, CV        : LongInt;
Begin
  If (V[i]=0) And (Await[i]>eps) Then Begin GetRTime:=Await[i]; Exit End;
  S0 := V[i];
  S1 := 0;
  For j:=1 To N Do
    If Gr[i,j] And R[j] Then
      Begin
        S0 := S0 - W[i,j]/G[i,j];
        S1 := S1 + (1/G[i,j])
      End;
  If S1 < eps Then Begin GetRTime := 1e20; Exit End;
  If S0 < eps Then Ti:=0 Else Ti:=S0/S1;
  Tt := 0;
  For k:=1 To N Do CW[k] := W[j,k];
  CV := V[i];
  Repeat
    Tt := Tt + Ti;
    RT := 1e20;
    For j:=1 To N Do
      If Gr[i,j] And R[j] Then
        Begin
          RD := Round (Int (eps*10 + (Ti + CW[j])/G[i,j]));
          CV := CV - RD;
          CW[j] := CW[j] + (Ti-RD*G[i,j]);
          If (G[i,j]-CW[j]) <10*eps Then
            Begin Dec(CV); CW[j]:=0 End;
          If CW[j]>eps Then If RT>G[i,j]-CW[j] Then RT:=G[i,j]-CW[j]
        End;
    If RT = 1e20 Then Rt := 0.05;
    If CV>0 Then Ti:=RT
  Until CV<=0;
  GetRTime := Tt
End;

Procedure TimePasses(i : Integer; Ti : Float);
Var
  j     : Integer;
  RD    : LongInt;
Begin
  If Await[i]>0 Then
    Begin
      Await[i] := Await[i] - Ti;
      If Await[i] < eps Then
         Await[i]:=0;
      Exit
    End;
  For j:=1 To N Do
    If Gr[i,j] And R[j] Then
        Begin
          RD := Round (Int ((Ti + W[i,j])/G[i,j]));
          V[i] := V[i] - RD;
          W[i,j] := W[i,j] + (Ti-RD*G[i,j]);
        End;
  If V[i]<0 Then V[i]:=0;
  If V[i]=0 Then Await[i]:=T;
End;



Procedure MakeStep;
Var
  i : Integer;
  Ti, Bt      : Float;
Begin
  Bt := 1e20;
  For i:=1 To N Do
    If Not R[i] Then
      Begin
        Ti := GetRTime (i);
        If Ti < Bt Then Bt:=Ti
      End;
  For i:=1 To N Do
    If Not R[i] Then
      TimePasses (i, Bt);
  Tm := Tm + Bt
End;

Procedure MakeOutput;
Begin
  Assign (Output, 'OUTPUT.TXT');
  ReWrite (Output);
  WriteLn (Tm-T:1:13);
  WriteLn (Lst)
End;

Begin
  MakeInput;
  InitMonsters;
  While Not All Do MakeStep;
  MakeOutput;
End.



























