Unit BigInts;

Interface

Const BigLen = 100;
      BigMod = 10000;
      LgBigMod = 4;

Type BigInt = Array [0..BigLen] Of Integer;

Procedure InitBig(Var A : BigInt; x : Integer);
Procedure AddInt(Var A : BigInt; x : Integer);
Procedure SubInt(Var A : BigInt; x : Integer);
Procedure MultInt(Var A : BigInt; x : Integer);
Procedure DivInt(Var A : BigInt; x : Integer);
Procedure AddBig(Var A : BigInt; Const B : BigInt);
Procedure SubBig(Var A : BigInt; Const B : BigInt);
Procedure MultBig(Var A : BigInt; Const B : BigInt);
{Procedure MultBigBig(Var A : BigInt; Const B,C : BigInt);}
Procedure BinomBigInt(Var A : BigInt; B : BigInt; x : Integer);
Procedure PrintBig(Const A : BigInt);

Implementation

Procedure Error(Const S : String);
Begin
  WriteLn('訡  믮 㭪樨 ',S);
  Halt(1);
End;

Procedure InitBig(Var A : BigInt; x : Integer);
Begin
  FillChar(A,SizeOf(A),0);
  A[0]:=1;
  A[1]:=x;
End;

Procedure AddInt(Var A : BigInt; x : Integer);
Var i,c : Integer;
Begin
  i:=0; c:=x;
  While c>0 Do
  Begin
    Inc(i); If i>BigLen Then Error('AddInt');
    Inc(A[i],c);
    c:=A[i] Div BigMod;
    A[i]:=A[i] Mod BigMod;
  End;
  If i>A[0] Then A[0]:=i;
End;

Procedure AddBig(Var A : BigInt; Const B : BigInt);
Var i,c : Integer;
Begin
  c:=0;
  If A[0]<B[0] Then A[0]:=B[0];
  For i:=1 To B[0] Do
  Begin
    Inc(A[i],B[i]+c);
    c:=A[i] Div BigMod;
    A[i]:=A[i] Mod BigMod;
  End;
  If c>0 Then
  Begin
    If A[0]=BigLen Then Error('AddBig');
    Inc(A[0]); A[A[0]]:=c;
  End;
End;

Procedure SubInt(Var A : BigInt; x : Integer);
Var i,c : Integer;
Begin
  i:=0; c:=x;
  While c>0 Do
  Begin
    Inc(i); If i>A[0] Then Error('SubInt');
    Inc(A[i],BigMod-c);
    c:=1-A[i] Div BigMod;
    A[i]:=A[i] Mod BigMod;
  End;
  If (A[0]>1) And (A[A[0]]=0) Then Dec(A[0]);
End;

Procedure SubBig(Var A : BigInt; Const B : BigInt);
Var i,c : Integer;
Begin
  If B[0]>A[0] Then Error('SubBig');
  c:=0;
  For i:=1 To B[0] Do
  Begin
    Inc(A[i],BigMod-B[i]-c);
    c:=1-A[i] Div BigMod;
    A[i]:=A[i] Mod BigMod;
  End;
  While c>0 Do
  Begin
    Inc(i); If i>A[0] Then Error('SubBig');
    Inc(A[i],BigMod-c);
    c:=1-A[i] Div BigMod;
    A[i]:=A[i] Mod BigMod;
  End;
  While (A[0]>1) And (A[A[0]]=0) Do Dec(A[0]);
End;

Procedure MultInt(Var A : BigInt; x : Integer);
Var i,c : Integer;
    t : LongInt;
Begin
  c:=0;
  For i:=1 To A[0] Do
  Begin
    t:=LongInt(A[i])*x+c;
    c:=t Div BigMod;
    A[i]:=t Mod BigMod;
  End;
  If c>0 Then
  Begin
    If i=BigLen Then Error('MultInt');
    Inc(i); A[i]:=c;
  End;
  A[0]:=i;
End;

Procedure MultBigBig(Var A : BigInt; Const B,C : BigInt);
Var i,j,cc : Integer;
    t : LongInt;
Begin
  If B[0]+C[0]-1>BigLen Then Error('MultBigBig');
  InitBig(A,0);
  For i:=1 To B[0] Do
  Begin
    cc:=0;
    For j:=1 To C[0] Do
    Begin
      t:=A[i+j-1]+LongInt(B[i])*C[j]+cc;
      cc:=t Div BigMod;
      A[i+j-1]:=t Mod BigMod;
    End;
    While cc>0 Do
    Begin
      Inc(j);
      If i+j-1>BigLen Then Error('MultBigBig');
      t:=A[i+j-1]+cc;
      cc:=t Div BigMod;
      A[i+j-1]:=t Mod BigMod;
    End;
    If i+j-1>A[0] Then A[0]:=i+j-1;
  End;
End;

Procedure MultBig(Var A : BigInt; Const B : BigInt);
Var P : BigInt;
Begin
  P:=A;
  MultBigBig(A,P,B);
End;

Procedure DivInt(Var A : BigInt; x : Integer);
Var i,c : Integer;
    t : LongInt;
Begin
  c:=0;
  For i:=A[0] DownTo 1 Do
  Begin
    t:=LongInt(c)*BigMod+A[i];
    A[i]:=t Div x;
    c:=t Mod x;
  End;
  If c<>0 Then Error('DivInt');
  If (A[0]>1) And (A[A[0]]=0) Then Dec(A[0]);
End;

Procedure BinomBigInt(Var A : BigInt; B : BigInt; x : Integer);
Var i : Integer;
Begin
  InitBig(A,1);
  For i:=1 To x Do
  Begin
    MultBig(A,B);
    SubInt(B,1);
  End;
  For i:=1 To x Do DivInt(A,i);
End;

Procedure PrintBig(Const A : BigInt);
Var i : Integer;
    S : String;
Begin
  Write(A[A[0]]);
  For i:=A[0]-1 DownTo 1 Do
  Begin
    Str(A[i],S);
    While Length(S)<LgBigMod Do S:='0'+S;
    Write(S);
  End;
End;

End.