program macro;
{$N-,E-}
uses strings;
{$DEFINE ERRFILE}

const
 TBSize         = 1024;         { input buffer size }
 MaxIdents      = 1024;         { max identifiers }

type
  LexType       = (_Null, _EOF, _Error,
                   _Ident, _IntConst, _StrConst, _Comment, _BoolConst,
                   _Semicolon, _R,
                   _OpBr, _ClBr, _OpBrc, _ClBrc,
                   _Plus, _Minus, _Times, _Divide,
                   _Eq, _Le, _Gr, _Leq, _Geq, _At, _Neq,

                   _SetPlus, _SetMinus, _SetMul,
                   _Ord, _Pred, _Succ, _Sequence, _Of,
                   _Optional, _Set, _Multi, _Define,
                   _Type, _Constant, _Mod,
                   _Integer, _String, _Char, _Boolean,
                   _Or, _And, _Not, _Chr, _False, _True
                  );
  NT            = (__null, __integer, __string, __char, __boolean,
                   __list, __listnode,
                   __undefined, __limited, __sequence, __set, __multiset
                   );
  Node          = ^TNode;
  TNode         = record
                    T             : NT;
                    ist           : boolean;
                    val           : integer;
                    ptr1, ptr2    : Node
                  end;
  string8       = string[26];
  iddescr       = record
                    S      : string8;
                    T      : byte;
                    P      : Node;
                  end;

var
  TB                            : array [0..TBSize-1] of char;
  idents                        : array [1..MaxIdents] of iddescr;
  defid                         : integer;
  TP, TL                        : integer;
  Fin                           : file;
  InFname, CurF                 : string;
  line, P, prline, prpos        : integer;
  thS, prS                      : string;

  clex                          : lextype;
  cval                          : integer;
  cstr                          : string;

procedure usagemsg;
begin
  writeln;
  writeln ('Usage: ');
  writeln (#9,paramstr(0),' <input file>[.ddl]');
  writeln;
  halt
end;

procedure err (N : integer; S : string; t : integer);
var
  ln, ps, i        : integer;
  St               : string;
begin
  if inFname = '' then t := -1;
  case t of
  1: begin ln:=line; ps:=P; St:=thS end;
  0: begin ln:=prline; ps:=prpos; St:=prS end
  else ln:=0
  end;
  if ln > 0 then
    begin
      writeln (inFname,'(',ln,'): Error ',N,': ',S);
      writeln (St);
      for i:=1 to pred(ps) do
        if St[i]=#9 then write (#9) else write (' ');
      writeln ('^')
    end
  else
    begin
      writeln ('Error ',N,': ',S);
    end;
{$IFDEF ERRFILE}
  delete (infname, pos('.',infname), 255);
  assign (output, infname + '.err');
  rewrite (output);
  if ln > 0 then
    begin
      writeln (inFname,'(',ln,'): Error ',N,': ',S);
      writeln (St);
      for i:=1 to pred(ps) do
        if St[i]=#9 then write (#9) else write (' ');
      writeln ('^')
    end
  else
    begin
      writeln ('Error ',N,': ',S);
    end;
{$ENDIF}
  halt
end;

procedure error (x : integer; S : string);
begin
{
 1  Out of memory
 2  Identifier expected
 3  Unknown identifier
 4  Duplicate identifier
 5  Syntax error
 7  Error in integer constant
 8  String constant exceeds line
10  Unexpected end of file
11  Line too long
12  Type identifier expected
13  Too many open files
14  Invalid file name
15  File not found
17  Invalid compiler directive
18  Too many files
20  Variable identifier expected
21  Error in type
22  Structure too large
23  Set base type out of range
25  Invalid string length
26  Type mismatch
27  Invalid subrange base type
28  Lower bound greater than upper bound
29  Ordinal type expected
30  Integer constant expected
31  Constant expected
32  Integer or real constant expected
33  Pointer Type identifier expected
34  Invalid function result type
35  Label identifier expected
36  BEGIN expected
37  END expected
38  Integer expression expected
39  Ordinal expression expected
40  Boolean expression expected
41  Operand types do not match
42  Error in expression
43  Illegal assignment
54  OF expected
56  Invalid relocatable reference
59  Undefined forward
61  Invalid typecast
63  Invalid file type
66  String variable expected
67  String expression expected
71  Internal stack overflow
74  Constant and case types do not match
75  Record or object variable expected
76  Constant out of range
77  File variable expected
78  Pointer expression expected
82  Undefined label in preceding statement part
83  Invalid @ argument
85  ";" expected
86  ":" expected
87  "," expected
88  "(" expected
89  ")" expected
90  "=" expected
91  ":=" expected
92  "[" or "(." expected
93  "]" or ".)" expected
94  "." expected
95  ".." expected
96  Too many variables
98  Integer variable expected
}
  case x of
  1:  err (x, 'Out of memory.', 0);
  2:  err (x, 'Identifier expected', 0);
  3:  err (x, 'Unknown identifier.', 0);
  4:  err (x, 'Duplicate identifier ('+S+').', 0);
  5:  err (x, 'Syntax error.', 0);
  6:  err (x, 'Error in constant', 0);
  8:  err (x, 'String constant exceeds line.', 1);
  11: err (x, 'Line too long.', 1);
  15: err (x, 'File not found ('+S+').', -1);
  19: err (x, 'Identifier buffer capacity exceeded.',0);
  26: err (x, 'Type mismatch.',0);
  29: err (x, 'Ordinal type expected',0);
  31: err (x, 'Constant expected.',0);
  38: err (x, 'Integer expression expected.',1*0);
  39: err (x, 'Ordinal expression expected.',1*0);
  40: err (x, 'Boolean expression expected.',1*0);
  41: err (x, 'Operand types do not match.', 1*0);
  42: err (x, 'Error in expression.',1*0);
  50: err (x, 'Invalid character encountried.', 1*0);
  51: err (x, 'Inexpected end of file encountred', 1*0);
  52: err (x, '''DEFINE'' expected.', 0);
  53: err (x, '''TYPE'' or ''CONSTANT'' expected.', 0);
  57: err (x, '''SET'' expected.', 0);
  58: err (x, 'Lists must be of same length.', 1*0);
  59: err (x, 'Undefined forward ('+S+').', 1*0);
  61: err (x, 'Invalid typecast', 0);
  62: err (x, 'Division by zero', 1*0);
  76: err (x, 'Constant out of range', 0);
  79: err (x, 'Integer expression expected', 1*0);
  88: err (x, '"(" expected', 0);
  89: err (x, '")" expected', 0);
  90: err (x, '"=" expected', 0);
  96: err (x, 'Too many variables.',0)
  else err (0, 'General error', 1*0)
  end
end;

procedure errorn (x : integer);
begin
  error (x, '')
end;


procedure parse;
var i : integer;
begin
  if paramcount = 0 then infname := 'program' else infname:=paramstr(1);
  if ParamCount > 1 then usagemsg;
  CurF    := '';
  line := 0;  P := 0;  prline := 0; prpos := 0;
  if pos('/',InFname) > 0 then usagemsg;
  if pos('.',InFname) = 0 then InFname := InFname + '.ddl';
  for i:=1 to length(InFname) do InFname[i] := UpCase (InFname[i]);
end;

function readchar : char;
begin
  if TL = 0 then
    begin
      readchar := #26;
      exit
    end;
  if TP >= TL then
    begin
      TP := 0;
      blockread (Fin, TB, TBSize, TL)
    end;
  if TP < TL then
    begin readchar := TB[TP]; inc(TP) end
  else readchar := #26
end;

procedure readline;
var i  : integer;
    C  : char;
begin
  inc (line);
  thS := '';
  P := 0;
  repeat
    C := readchar;
    if P > 239 then errorn (11);
    if C <> #13 then
      begin
        inc (P);
        ThS[P] := C;
        inc (ThS[0])
      end
  until C in [#13,#26];
  inc (P);  ThS[P] := #0;
  P := 1;
  if C = #13 then if readchar<>#10 then errorn (-1);
end;

procedure openfile;
begin
{$I-}
  assign (Fin, InFname);
  if ioresult <> 0 then error (15, InFname);
  reset (Fin, 1);
  if ioresult <> 0 then error (15, InFname);
{$I+}
  TP := TBSIZE;
  TL := TP;
  defid := 0;
  readline
end;

procedure skipspc;
begin
  repeat
    while thS[P] in [#9,' '] do inc (P);
    if thS[P] <> #0 then exit;
    readline
  until false;
end;

function lowc (C : char) : char;
begin
  case C of
  'A'..'Z'   : C := chr(ord(C) + $20);
  #$80..#$8f : C := chr(ord(C) + $20);
  #$90..#$9f : C := chr(ord(C) + $50)
  end;
  lowc := C
end;

function identnum (S : string8) : integer;
var i : integer;
begin
  for i:=1 to defid do
    if S = idents[i].S then
      begin identnum:=i; exit end;
  if defid = MaxIdents then errorn (96);
  inc (defid);
  idents[defid].S := S;
  idents[defid].P := nil;
  idents[defid].T := 0;
  identnum := defid
end;

procedure _getlexem;
var t : longint;
    C : char;
begin
  skipspc;
  clex := _Error;
  cstr := '';
  cval := 0;
  prS := thS;  prline := line;  prpos := P;
  case thS[P] of
  #26: begin clex:=_EOF; exit end;
  'A'..'Z','a'..'z','_','?','.','$',#$80..#$AF,#$E0..#$EF:
    begin
      clex := _Ident;
      repeat
        cstr := cstr + lowc(thS[P]);
        inc (P)
      until not (ThS[P] in
        ['0'..'9','A'..'Z','a'..'z','_',#$80..#$AF,#$E0..#$EF,'$','_','?']);
      if ThS[P] = '.' then errorn (50);
      if cstr = 'ord'       then clex := _Ord        else
      if cstr = 'pred'      then clex := _Pred       else
      if cstr = 'succ'      then clex := _Succ       else
      if cstr = 'sequence'  then clex := _Sequence   else
      if cstr = 'of'        then clex := _Of         else
      if cstr = 'optional'  then clex := _Optional   else
      if cstr = 'set'       then clex := _Set        else
      if cstr = 'multi'     then clex := _Multi      else
      if cstr = 'define'    then clex := _Define     else
      if cstr = 'type'      then clex := _Type       else
      if cstr = 'constant'  then clex := _Constant   else
      if cstr = 'mod'       then clex := _Mod        else
      if cstr = 'integer'   then clex := _Integer    else
      if cstr = 'string'    then clex := _String     else
      if cstr = 'char'      then clex := _Char       else
      if cstr = 'boolean'   then clex := _Boolean    else
      if cstr = 'or'        then clex := _Or         else
      if cstr = 'and'       then clex := _And        else
      if cstr = 'not'       then clex := _Not        else
      if cstr = 'chr'       then clex := _Chr        else
      if cstr = 'false'     then clex := _False      else
      if cstr = 'true'      then clex := _True       else
      if cstr = 'plus'      then clex := _SetPlus    else
      if cstr = 'minus'     then clex := _SetMinus   else
      if cstr = 'mul'       then clex := _SetMul     else
      cval := identnum (cstr);
      if clex = _False then begin clex := _Boolconst; cval := 0 end;
      if clex = _True  then begin clex := _Boolconst; cval := 1 end;
    end;
  '0'..'9':
    begin
      clex := _IntConst;
      t := 0;
      repeat
        t := t*10 + (ord(thS[P]) - 48);
        inc (P);
        if t > 32768 then errorn (76)
      until not (thS[P] in ['0'..'9']);
      cval := t
    end;
  '"', '''':
    begin
      C := thS[P];
      clex := _StrConst;
      cstr := '';
      repeat
        inc (P);
        while (thS[P]<>#0) and (thS[P]<>C) do
          begin cstr:=cstr+thS[P]; inc(P) end;
        if thS[P]=#0 then errorn(8);
        inc (P);
        if thS[P]=C then cstr := cstr + C
      until thS[P] <> C
    end;
  ';' : begin clex:=_Semicolon; inc(P) end;
  ',' : begin clex:=_R;         inc(P) end;
  '(' : begin clex:=_OpBr;      inc(P) end;
  ')' : begin clex:=_ClBr;      inc(P) end;
  '{' : begin clex:=_OpBrc;     inc(P) end;
  '}' : begin clex:=_ClBrc;     inc(P) end;
  '+' : begin clex:=_Plus;      inc(P) end;
  '-' : begin clex:=_Minus;     inc(P) end;
  '*' : begin clex:=_Times;     inc(P) end;
  '=' : begin clex:=_Eq;        inc(P) end;
  '@' : begin clex:=_At;        inc(P) end;
  '/' : begin
          clex:=_Divide;  inc(P);
          if thS[P] = '/' then
            begin
              clex := _Comment;
              repeat inc(P) until ThS[P] = #0
            end
        end;
  '<' : begin
          clex:=_Le;  inc(P);
          if thS[P] = '>' then begin clex:=_Neq; inc(P) end else
          if thS[P] = '=' then begin clex:=_Leq; inc(P) end
        end;
  '>' : begin
          clex:=_Gr;  inc(P);
          if thS[P] = '=' then begin clex:=_Geq; inc(P) end
        end
  else clex := _Error
  end;
end;

procedure getlexem;
begin
  repeat _getlexem until clex <> _Comment;
  if clex = _Error then errorn (50)
end;

function getident : integer;
begin
  getlexem;
  if clex <> _Ident then errorn (2);
  getident := cval;
end;

{ --------------- end lexical analyser -------------- }

function newnode : Node;
Var N : Node;
begin
  new (N);
  newnode := N;
  with N^ do
    begin
      ist := false;
      T := __null;
      ptr1 := nil;
      ptr2 := nil;
      val := 0
    end
end;

function newtype (u : NT) : Node;
var N : Node;
begin
  N := newnode;
  N^.ist := true;
  N^.T := u;
  newtype := N
end;

function emptylist : Node;
var N : Node;
begin
  N := newnode;
  emptylist := N;
  N^.T := __list
end;

procedure addlist (N, E : Node);
var
  N1     : Node;
begin
  N1 := newnode;
  N1^.T := __listnode;
  N1^.ptr1 := E;
  if N^.ptr2 = nil then
    begin
      N^.ptr1 := N1;
      N^.ptr2 := N1;
      N^.val := 1;
      exit
    end;
  inc (N^.val);
  N^.ptr2^.ptr2 := N1;
  N^.ptr2 := N1
end;


procedure c_crefs (N : integer; Pr : Node);
begin
  idents[N].P^ := Pr^;
  idents[N].T := idents[N].T and $7f
end;

function c_type : Node; forward;

function c_basictype : Node;
var N : Node;
begin
  case clex of
  _Integer:  N := newtype(__Integer);
  _Char:     N := newtype(__Char);
  _Boolean:  N := newtype(__Boolean);
  _string:   N := newtype(__String)
  else errorn(61)
  end;
  getlexem;
  c_basictype := N;
end;

function c_identtype : Node;
var N : Node;
begin
  if idents[cval].T in [1,$81] then
    N := idents[cval].P else
  if idents[cval].T = 0 then
    begin
      idents[cval].T := $81;
      N := newtype (__Undefined);
      N^.val := cval
    end
  else errorn(4);
  getlexem;
  c_identtype := nil;
end;

function c_enumtype : Node;
begin
  c_enumtype := nil;
end;

function c_seqtype : Node;
var
  N          : Node;
  B          : Boolean;
begin
  getlexem;
  if clex = _of then
    begin
      getlexem;
      N := newtype (__Sequence);
      N^.val := -1;
      N^.ptr1 := c_type;
      c_seqtype := N;
      exit
    end;
  if clex <> _OpBr then errorn (5);
  getlexem;
  N := newtype (__Sequence);
  repeat
    B := (clex = _Optional);
    if B then getlexem;
    addlist (N, c_type);
    N^.ptr2^.ist := B;
    if not (clex in [_R, _ClBr]) then errorn (5);
    B := clex = _ClBr;
    getlexem
  until B;
  c_seqtype := N;
end;

function c_settype : Node;
var
  N          : Node;
  B          : Boolean;
begin
  B := clex = _multi;
  if B then begin getlexem; if clex<>_set then errorn(57) end;
  getlexem;
  if clex = _of then
    begin
      getlexem;
      N := newtype (__Set);
      if B then N^.T := __multiset;
      N^.val := -1;
      N^.ptr1 := c_type;
      c_settype := N;
      exit
    end;
  if clex <> _OpBr then errorn (5);
  getlexem;
  N := newtype (__MultiSet);
  repeat
    B := (clex = _Optional);
    if B then getlexem;
    addlist (N, c_type);
    N^.ptr2^.ist := B;
    if not (clex in [_R, _ClBr]) then errorn (5);
    B := clex = _ClBr;
    getlexem
  until B;
  c_settype := N;
end;

function c_type : Node;
begin
  c_type := nil;
  case clex of
  _Integer, _Char, _Boolean, _String :
            c_type := c_basictype;
  _Ident :  c_type := c_identtype;
  _OpBr  :  c_type := c_enumtype;
  _Sequence:
            c_type := c_seqtype;
  _Set, _Multi:
            c_type := c_settype
  else
    begin
      errorn (5)
    end
  end;

end;

var
  BBB : array [0..3] of char;
  U1, U2, U    : PChar;

function oper (op : lextype; N1, N2 : Node) : Node;
var
  N, P1, P2    : Node;
  i            : integer;
  tmp          : longint;
begin
  N := nil;
  if (N1<>nil) and (N2<>nil) and (N1^.T=__list) and (N2^.T=__list) then
    if op <> _at then
      begin
        if N1^.val <> N2^.val then errorn (58);
        N := emptylist;
        P1 := N1^.ptr1;
        P2 := N2^.ptr1;
        for i:=1 to N1^.val do
          begin
            addlist (N, oper (op, P1^.ptr1, P2^.ptr1));
            P1 := P1^.ptr2;
            P2 := P2^.ptr2
          end;
        oper := N;
        exit
      end;
  if (N1<>nil) and (N2=nil) and (N1^.T=__list) then
    if op <> _at then
      begin
        N := emptylist;
        P1 := N1^.ptr1;
        for i:=1 to N1^.val do
          begin
            addlist (N, oper (op, P1^.ptr1, nil));
            P1 := P1^.ptr2;
          end;
        oper := N;
        exit
      end;

  case op of
  _Minus, _Times, _Divide, _Mod:
    begin
      if N1=nil then errorn (6);
      if (N2=nil) and (op<>_Minus) then errorn(6);
      if (N1^.T<>__Integer) or ((N2<>nil) and (N2^.T<>__Integer)) then errorn(79);
      N := newnode;
      N^.T := __Integer;
      case op of
      _Minus : if N2=nil then tmp := - N1^.val else
                              tmp := N1^.val - N2^.val;
      _Times : tmp := N1^.val * N2^.val;
      _Divide : if N2^.val = 0 then errorn(62) else
                tmp := N1^.val div N2^.val;
      _Mod   : if N2^.val = 0 then errorn(62) else
                tmp := N1^.val mod N2^.val;
      end;
      if (tmp<-32768) or (tmp>32767) then errorn (76);
      N^.val:= tmp
    end;
  _Plus :
    begin
      if (N1=nil) or (N2=nil) then errorn(6);
      if (N1^.T=__Integer) and (N2^.T=__Integer) then
        begin
          N:=newnode;
          N^.T := __Integer;
          tmp := N1^.val + N2^.val;
          if (tmp<-32768) or (tmp>32767) then errorn (76);
          N^.val:= tmp
        end
      else if (N1^.T in [__Char,__String]) and (N2^.T in [__Char,__String]) then
        begin
          N:=newnode;
          N^.T := __String;
          if N1^.T=__String then U1:=pchar(N1^.ptr1)
                            else begin BBB[0]:=chr(N1^.val); BBB[1]:=#0; U1:=@BBB[0] end;
          if N2^.T=__String then U2:=pchar(N2^.ptr1)
                            else begin BBB[2]:=chr(N2^.val); BBB[3]:=#0; U2:=@BBB[2] end;
          getmem (U, strlen(U1) + strlen(U2) + 1);
          strcopy (U, U1);
          strcat (U, U2);
          N^.ptr1 := node(U)
        end
      else errorn(79)
    end;
  _ord :
    begin
      if (N1=nil) or (N1^.T=__String) then errorn(6);
      N:=newnode;
      N^.val := N1^.val;
      N^.T := __Integer
    end;
  _chr :
    begin
      if (N1=nil) or (N1^.T<>__Integer) then errorn(6);
      N:=newnode;
      N^.val := N1^.val;
      N^.T := __Char
    end;
  _pred, _succ:
    begin
      if (N1=nil) or (N1^.T=__String) then errorn(6);
      N := newnode;
      N^ := N1^;
      if op=_pred then dec(N^.val) else inc(N^.val);
      if N^.T = __Integer then i:=-1 else
      if N^.T = __Char then i:=256 else
      if N^.T = __Boolean then i:=2 else
      if N^.ptr1 <> nil then i:=N^.ptr1^.val;
      if N^.val >= i then N^.val := N^.val - i else
      if N^.val < 0 then N^.val := N^.val + i;
    end;
  _and, _or, _not:
    begin
      if (N1=nil) or (N1^.T<>__Boolean) or (((N2=nil) or (N2^.T<>__Boolean)) and (op<>_Not)) then
        errorn (40);
      N := newnode;
      N^.T := __Boolean;
      case op of
      _and : N^.val := ord((N1^.val<>0) and (N2^.val<>0));
      _or  : N^.val := ord((N1^.val<>0) or (N2^.val<>0));
      _not : N^.val := ord(not (N1^.val<>0));
      end;
  end;
  _le, _gr, _leq, _geq, _eq, _neq:
    begin
      if (N1=nil) or (N2=nil) then errorn(6);
      if (N1^.T <> N2^.T) then
        errorn (41);
      if (N1^.T = __String) then
          i := strcomp (pchar(N1^.ptr1), pchar(N2^.ptr2))
      else i := N1^.val - N2^.val;
      case op of
      _le :  i := ord (i<0);
      _gr :  i := ord (i>0);
      _eq :  i := ord (i=0);
      _leq:  i := ord (i<=0);
      _geq:  i := ord (i>=0);
      _neq:  i := ord (i<>0);
      end;
      N := newnode;
      N^.T := __Boolean;
      N^.val := i
    end
  else errorn (6)
  end;
  oper := N;
end;


function cc9 : Node; forward;


function cc2 : Node;
var N : Node;
    i : integer;
    op : lextype;
begin
  case clex of
  _OpBr     : begin getlexem; N:=cc9;
                    if clex <> _ClBr then errorn (89);
                    getlexem
              end;
  _IntConst : begin N := newnode;
                    N^.T := __Integer;
                    N^.val := cval;
                    getlexem
              end;
  _BoolConst: begin N := newnode;
                    N^.T := __Boolean;
                    N^.val := cval;
                    getlexem
              end;
  _StrConst : if length (cstr) <> 1 then
              begin N := newnode;
                    N^.T := __String;
                    getmem (N^.ptr1, length(cstr)+1);
                    for i:=1 to length(cstr) do
                      pchar(N^.ptr1)[i-1] := cstr[i];
                    pchar(N^.ptr1)[length(cstr)] := #0;
                    getlexem
              end else
              begin N := newnode;
                    N^.T := __Char;
                    N^.val := ord (cstr[1]);
                    getlexem
              end;
  _Ident    : begin if idents[cval].T <> 2 then errorn (31);
                    N := idents[cval].P;
                    getlexem
              end;
  _OpBrc    : begin N := emptylist;
                    getlexem;
                    if clex <> _ClBrc then
                      begin
                        repeat
                          addlist (N, cc9);
                          if not (clex in [_R, _ClBrc]) then errorn(5);
                          if clex=_R then i:=1 else i:=0;
                          getlexem
                        until i=0
                      end
                    else getlexem
              end;
  _Pred, _Succ, _Ord, _Chr :
              begin
                op := clex;
                getlexem;
                if clex <> _OpBr then errorn (88);
                getlexem;
                N := oper (op, cc9, nil);
                if clex <> _ClBr then errorn (89);
                getlexem;
              end;
  _Minus:     begin
                getlexem;
                N := oper (_Minus, cc2, nil)
              end;
  else errorn(5)
  end;
  cc2 := N
end;



function cc3 : Node;
var
  N          : Node;
begin
  N := cc2;
  while clex = _at do
    begin
      getlexem;
      N := oper (_at, N, cc2)
    end;
  cc3 := N
end;

function cc4 : Node;
var
  N          : Node;
  T          : lextype;
begin
  N := cc3;
  while clex in [_times, _divide, _mod, _setmul] do
    begin
      T := clex;
      getlexem;
      N := oper (T, N, cc3)
    end;
  cc4 := N
end;

function cc5 : Node;
var
  N          : Node;
  T          : lextype;
begin
  N := cc4;
  while clex in [_plus, _minus, _setplus, _setminus] do
    begin
      T := clex;
      getlexem;
      N := oper (T, N, cc4)
    end;
  cc5 := N
end;

function cc6 : Node;
var
  N          : Node;
  T          : lextype;
begin
  N := cc5;
  while clex in [_le, _eq, _gr, _leq, _neq, _geq] do
    begin
      T := clex;
      getlexem;
      N := oper (T, N, cc5)
    end;
  cc6 := N
end;


function cc7 : Node;
begin
  if clex = _not then begin getlexem; cc7 := oper (_not, cc7, nil) end
                 else cc7 := cc6
end;

function cc8 : Node;
var
  N      : Node;
begin
  N := cc7;
  while clex = _and do
    begin
      getlexem;
      N := oper (_and, N, cc7);
    end;
  cc8 := N
end;

function cc9 : Node;
var
  N      : Node;
begin
  N := cc8;
  while clex = _or do
    begin
      getlexem;
      N := oper (_or, N, cc8);
    end;
  cc9 := N
end;

function c_const : Node;
begin
  c_const := cc9;
end;

procedure c_prd;
var R : lextype;
    I : integer;
begin
  if clex <> _Define then errorn (52);
  getlexem;
  R := clex;
  if not (R in [_Type, _Constant]) then errorn (53);
  I := getident;
  getlexem;
  if clex <> _Eq then errorn (90);
  getlexem;
  if R = _Type then
    begin
      if not (idents[I].T in [0,$81]) then error (4, idents[I].S);
      if idents[I].T = $81 then c_crefs (I, c_type)
                           else idents[I].P := c_type;
      idents[I].T := 1;
    end
  else
    begin
      if not (idents[I].T in [0,$82]) then error (4, idents[I].S);
      if idents[I].T = $82 then c_crefs (I, c_const)
                           else idents[I].P := c_const;
      idents[I].T := 2;
    end;
  if not (clex in [_EOF, _Semicolon]) then errorn (5)
end;

procedure c_program;
begin
  getlexem;
  c_prd;
  while clex=_semicolon do
    begin
      getlexem;
      if clex = _EOF then exit;
      if clex <> _semicolon then c_prd
    end;
  if clex <> _EOF then errorn (51)
end;

procedure testforw;
var i : integer;
begin
  for i:=1 to defid do
    if idents[i].T > $80 then error (59, idents[i].S)
end;

{--------------------------------------}

function _MT (C, T : Node) : boolean;
var
  P1, P2        : Node;
begin
  _MT := false;
  if (C=nil) or (T=nil) then exit;
  if (C^.T <> __list) then
    begin
      if (C^.ptr1<>nil) and (C^.ptr1=T) then
        begin _MT:=true; exit end;
      if (C^.T = T^.T) then begin _MT:=true; exit end;
      _MT := (C^.T = __char) and (T^.T = __string);
      exit
    end;
  if T^.T in [__sequence,__set,__multiset] then
    if T^.val = -1 then
      begin
        P2 := T^.ptr1;
        P1 := C^.ptr1;
        while P1 <> nil do
          begin
            if not _MT (P1^.ptr1, P2) then exit;
            P1 := P1^.ptr2
          end;
        _MT := true;
        exit
      end
    else
      begin
        P2 := T^.ptr1;
        P1 := C^.ptr1;
        while P1 <> nil do
          begin
            while (P2<>nil) and (not _MT(P1^.ptr1, P2^.ptr1)) do
              begin
                if not P2^.ist then exit;
                P2 := P2^.ptr2
              end;
            if P2=nil then exit;
            P2 := P2^.ptr2;
            P1 := P1^.ptr2
          end;
        while (P2<>nil) do
          begin
            if not P2^.ist then exit;
            P2 := P2^.ptr2
          end;
        _MT := true
      end;
end;

{--------------------------------------}

procedure swap (a, b : integer);
var t : iddescr;
begin
  t := idents[a];
  idents[a] := idents[b];
  idents[b] := t
end;

procedure qsort (a, b : integer);
var
  i, j   : integer;
  h      : string8;
begin
  if a>=b then exit;
  if b-a=1 then
    begin
      if idents[a].S > idents[b].S then swap(a,b);
      exit
    end;
  h := idents[(a+b) shr 1].S;
  i := a;
  j := b;
  repeat
    while idents[i].S<h do inc(i);
    while idents[j].S>h do dec(j);
    if i<j then begin swap(i,j); inc(i); dec(j); end;
  until i>=j;
  qsort (a, j);
  qsort (i, b)
end;

procedure writeconst (N : Node);
var
  P      : Node;
begin
  case N^.T of
  __Integer  : write (N^.val);
  __Char     : write (chr(N^.val));
  __Boolean  : if N^.val = 0 then write ('false') else write('true');
  __String   : begin U := pchar(N^.ptr1); write ('''');
                     while U^<>#0 do begin write(U^);
                     if U^='''' then write(U^); inc(U) end; write('''') end;
  __list     : if N^.val<=0 then write ('{}') else
               begin
                 P:=N^.ptr1;
                 write('{');
                 repeat
                   writeconst (P^.ptr1);
                   P := P^.ptr2;
                   if P <> nil then write (',')
                 until P = nil;
                 write ('}')
               end
  else write ('<INVALID>')
  end
end;

procedure _WT (N : Node);
var
  P      : Node;
begin
  case N^.T of
  __Integer  : write ('integer');
  __Char     : write ('char');
  __Boolean  : write ('boolean');
  __String   : write ('string');
  __list     : if N^.val<=0 then write ('sequence(optional boolean)') else
               begin
                 P:=N^.ptr1;
                 write('sequence(');
                 repeat
                   _WT (P^.ptr1);
                   P := P^.ptr2;
                   if P <> nil then write (',')
                 until P = nil;
                 write (')')
               end
  else write ('<INVALID>')
  end
end;

procedure makeoutput;
var i, j, k : integer;
begin
  qsort (1, defid);
  delete (infname, pos('.',infname), 255);
  assign (output, InFname+'.out'); rewrite(output);
  for i:=1 to defid do
    if idents[i].T=2 then
      begin
        write (idents[i].S,': ');
        k := 0;
        for j:=1 to defid do
          if idents[j].T=1 then
            if _MT (idents[i].P, idents[j].P) then
              begin
                if k > 0 then write (', ');
                write (idents[j].S);
                inc (k)
              end;
        if k = 0 then _WT (idents[i].P);
        write (' = ');
        writeconst (idents[i].P);
        writeln
      end
end;

begin
  parse;
  openfile;

  c_program;
  testforw;
  makeoutput;

end.