{$A+,B-,D+,E+,F-,G-,I+,L+,N-,O-,P-,Q-,R-,S+,T-,V+,X+}
{$M 65520,0,655360}

{Let us consider X and Y are two open lockers. The distance between
X and Y means the minimum number of the usage the key to open both X and Y.


I used minimum cost flow algorithm to match the closed lockers. Hungarian
algorithm should work faster.}




Program The_Lockers;
Const
     Max = 100;
Type
    Lock = Record
                 row,col : byte;
             end;
    LockAr = Array [1..Max div 2] of Lock;
    LockMat = Array[1..Max div 2,1..Max div 2] of byte;
    FlowMat = Array[1..Max,1..Max] of integer;
    PathAr = Array[1..Max] of byte;
Var
   Infile,OutFile : Text;
   M : integer;      {Number of the open lockers}
   Lockers : LockAr; {List of the open lockers}
   DisMat : LockMat; {Matrix of the distances between open lockers}
   CaseNo : integer;
{-------------------------------------------------}
Procedure ReadOneCase(Var Lockers: LockAr; var M:integer);
{Read the number of the open lockers to M and read the positions of the
open lockers to the array Lockers.
}
Var
   i:integer;
Begin
     Readln(InFile,M);
     For i:=1 to M Do
         Read(InFile,Lockers[i].Row,Lockers[i].Col);
     Readln(InFile);
End;
{-------------------------------------------------}
Function CalcDis(X,Y : Lock):integer;
{X and Y are two open lockers. Returns the distance between X and Y}
Var
   MaxValue : integer;
Begin
     MaxValue := Abs(X.Row - Y.Row);
     if Abs(X.Col - Y.Col) > MaxValue then
        MaxValue := Abs(X.Col - Y.Col);
     If MaxValue > 0 then
        CalcDis := MaxValue
     else CalcDis := Max;
End;
{-------------------------------------------------}
Procedure GetMat(Var DisMat : LockMat; Lockers: LockAr; M:integer);
{Calculates the distance any two open lockers and keeps them in the matrix
DisMat. Lockers keeps the list of the open lockers. M is the number of the
open lockers.
}
Var
   i,j:integer;
Begin
     For i:=1 to M do
         For j:=1 to M do
             DisMat[i,j] := CalcDis(Lockers[i],Lockers[j]);
End;
{-------------------------------------------------}
Procedure GetPaths(Var Paths,Flows:FlowMat; M:integer);
Var
   i,j:integer;
Begin
     For i:=1 to 2*M do
         For j:=1 to 2*M Do
             If Flows[i,j] = Max then
                Paths[i,j] := 0
             Else Paths[i,j] := i;
End;
{-------------------------------------------------}
Procedure InitialFlow(Var Flows:FlowMat; Var DisMat:LockMat; M:integer);
{Finds the initial solution.}
Var
   i,j:integer;
Begin
     For i:=1 to 2*M Do
         For j:=1 to 2*M Do
             Flows[i,j] := Max;

     For i:=1 to 2*M Do
         Flows[i,i] := 0;

     For i:=1 to M Do
         For j:= M+1 to 2*M Do
             Flows[i,j] := DisMat[i,j-M];

     For i:=1 to M-1 Do
     Begin
         Flows[i,M+i+1] := Max;
         Flows[M+i+1,i] := - DisMat[i+1,i];
     End;
     Flows[M,M+1] := Max;
     Flows[M+1,M] := -DisMat[1,M];
End;
{-------------------------------------------------}
Procedure FindCycle(Var Paths:FlowMat;Var NegCycle:PathAr; Var Count,i,j:integer);
Begin
     if Paths[i,j]=i then
        begin
             inc(Count);
             NegCycle[Count]:=i;
        end
     else
         begin
              FindCycle(Paths,NegCycle,Count,i,Paths[i,j]);
              FindCycle(Paths,NegCycle,Count,Paths[i,j],j);
         end;
End;
{-------------------------------------------------}
Function Floyd(Flows:FlowMat; Var NegCycle:PathAr;
               Var Count,M:integer):Boolean;
{Returns True if ther is a negative cycle. Returns False otherwise. In the
case there is a negative cycle, NegCycle keeps the nodes in the cycle and
count keeps number of the nodes in the cycle.}
Var
   Paths : FlowMat;
   i,j,k : integer;
Begin
     GetPaths(Paths,Flows,M);
     For k:=1 to 2*M do
         For i:=1 to 2*M do
             For j:=1 to 2*M do
                 If (Flows[i,k]<>Max) and (Flows[k,j]<>Max)
                    and (Flows[i,j] > Flows[i,k]+Flows[k,j]) then
                 Begin
                      Flows[i,j] := Flows[i,k] + Flows[k,j];
                      Paths[i,j] := k;
                      If (i=j) and (Flows[i,j]<0) then
                      Begin
                           Count:=0;
                           FindCycle(Paths,NegCycle,Count,i,i);
                           Count := Count + 1;
                           NegCycle[Count]:=i;
                           Floyd := True;
                           Exit;
                      End;
                 End;
     Floyd := False;
End;
{-------------------------------------------------}
Procedure ModifyFlows(Var Flows : FlowMat; Var NegCycle:PathAr;
                       Count:integer);
Var
   i:integer;
   OldValue:integer;
Begin
     For i:=1 to Count-1 Do
     Begin
          If Flows[NegCycle[i],NegCycle[i+1]]<Max then
          Begin
               OldValue := Flows[NegCycle[i],NegCycle[i+1]];
               Flows[NegCycle[i],NegCycle[i+1]] := Max;
               Flows[NegCycle[i+1],NegCycle[i]] := -OldValue;
          End;
     End;
End;
{-------------------------------------------------}
Function GetResult(Var Flows:FlowMat; M:integer):integer;
Var
     i,j,Res:integer;
Begin
     Res := 0;
     For i:=1 to 2*M Do
         For j:=1 to 2*M Do
             If Flows[i,j] < 0 then
                Res := Res - Flows[i,j];
     GetResult := Res;
End;
{-------------------------------------------------}
Function Result(Var DisMat:LockMat; M:integer):longint;
Var
   Flows: FlowMat;
   NegCycle : PathAr;
   Count : integer; {Number of the nodes in the negative cycle.}
Begin
     {Find initial Flow for the flow M}
     InitialFlow(Flows,DisMat,M);

     {While the current solution is not optimal}
     While Floyd(Flows,NegCycle,Count,M) do
           ModifyFlows(Flows,NegCycle,Count);

     {Return the result}
     Result := GetResult(Flows,M) div 2;
End;
{-------------------------------------------------}
Procedure WriteOneCase(CaseNo,Res:integer);
Begin
     Writeln(OutFile,'Case #',CaseNo,' : ',Res);
     Writeln(OutFile);
End;
{-------------------------------------------------}
Begin
     Assign(InFile,'lockers.in');
     Assign(OutFile,'lockers.out');
     Reset(InFile);
     ReWrite(OutFile);

     CaseNo:=0;
     ReadOneCase(Lockers,M);
     While M<>0 Do
     Begin
          CaseNo := CaseNo + 1;
          If M mod 2 = 1 then
          Begin
               Writeln(OutFile, 'Case #',CaseNo,' : Impossible');
               ReadOneCase(Lockers,M);
               Continue;
          End;
          GetMat(DisMat,Lockers,M);
          WriteOneCase(CaseNo,Result(DisMat,M));
          ReadOneCase(Lockers,M);
     End;
     Close(InFile);
     Close(OutFile);
End.


