#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <vector>
#include <iostream>

using namespace std;

#define xMax (1<<15)
#define NM 10000
#define file "dmg"
#define EPS 0.01
#define tip(x) cout << #x << " = " << (x) << endl
#define ALL(x) x.begin(), x.end()

struct point {
       double x,y;
       point() {}
       point(double a, double b) { x=a; y=b; };
};
bool operator < (const point &a, const point &b) {
     if ( abs (a.x-b.x) < EPS ) return a.y < b.y;
     else return a.x < b.x;
}
bool operator == (const point &a, const point &b) {
     return ( abs(a.x - b.x) < EPS ) && ( abs(a.y - b.y)<EPS );
}

int N, sol;
vector< point > p;
vector< double > ax,bx,cz;
double x[NM], y[NM];

bool lookup(double cx, double cy) {
       int i,j=0; 
       for (i=1; i<N; i<<=1) ;
       for (i>>=1; ; i>>=1)
       {
            if (i+j>=N) continue;
            if ( (abs( x[i+j] - cx) < EPS) && (abs(y[i+j] - cy) < EPS )) return true;
            if (abs( x[i+j] - cx) < EPS)
		{ if (y[i+j] < cy) j+=i; }
		else
		{ if (x[i+j] < cx) j+=i; }
            if (i==0) return false;
       }
}

void read_data() {
     scanf("%d", &N);
     p.resize(N);
     for(int i=0; i<N; ++i) {
	     double a, b;    	
             scanf("%lf%lf", &a,&b);
             p[i]= point (a,b) ;
     }
     sort( ALL(p) );
     for (int i=0; i<N ;i++) {
        x[i]=p[i].x;
	y[i]=p[i].y;
     }
}

#define Matrix double*
#define Ident {1,0,0,0,1,0,0,0,1}      // 0 3 6
void mult(Matrix a, const Matrix b){    // 1 4 7
     double t[9];                       // 2 5 8
     t[0] = a[0]*b[0] + a[1]*b[3] + a[2]*b[6] ;
     t[1] = a[0]*b[1] + a[1]*b[4] + a[2]*b[7] ;
     t[2] = a[0]*b[2] + a[1]*b[5] + a[2]*b[8] ;
     t[3] = a[3]*b[0] + a[4]*b[3] + a[5]*b[6] ;
     t[4] = a[3]*b[1] + a[4]*b[4] + a[5]*b[7] ;
     t[5] = a[3]*b[2] + a[4]*b[5] + a[5]*b[8] ;
     t[6] = a[6]*b[0] + a[7]*b[3] + a[8]*b[6] ;
     t[7] = a[6]*b[1] + a[7]*b[4] + a[8]*b[7] ;
     t[8] = a[6]*b[2] + a[7]*b[5] + a[8]*b[8] ;
     memcpy(a, t, sizeof(double)*9);
};
void Rotate(Matrix a, const double sin, const double cos) {
     double rot[9] = {cos, -sin, 0,
            sin, cos, 0,
            0,0,1 };

     mult(a, rot);
}
void Translate(Matrix a, const double x, const double y) {
     double trans[9] = {1,0,x,0,1,y,0,0,1};
     mult(a, trans);
}

void Scale(Matrix a, const double x, const double y) {
     double scale[9] = {x,0,0,0,y,0,0,0,1};
     mult(a,scale);
}

void GetSymmetric(const Matrix a,double &x, double &y) {
     double tx,ty;
     tx = a[0]*x+a[1]*y +a[2];
     ty = a[3]*x+a[4]*y +a[5];
     x=tx; y= ty;
}

bool check(Matrix mat) {
    for(int j=1; j<N; j++)
    {
        double cx=p[j].x, cy=p[j].y;
        GetSymmetric(mat, cx, cy);
//	cout << "The symmetric of " << x[j] << "," << y[j] << " is " << cx <<","<<cy<<"\n";
        if ( !lookup(cx, cy) )
           return false;
    }
    return true;
}

void debug_chk_mat(Matrix mat) {
    printf("%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.3lf\n\n", mat[0], mat[1], mat[2], mat[3], mat[4], mat[5], mat[6], mat[7], mat[8]);	         
    for (int i=0; i<N; i++)
    printf("When point (%3.2lf,%3.2lf) is multiplied with the matrix the point (%3.2lf,%3.2lf) is obtained \n", x[i], y[i], x[i]*mat[0] + y[i]*mat[1] + mat[2], 
														    x[i]*mat[3] + y[i]*mat[4] + mat[5]);
}

void solve() {

     double cx,cy;  // Obtain the gravity center, the symmetry axis must pass trough this point
     cx=cy=0;
     for (int i=0; i<N; ++i) {
        cx += x[i];
	cy += y[i];
     }
     cx /= N;
     cy /= N;
     double vx,vy, cos, sin;
//     tip(x[0]); tip(p[0].y);
     for(int i=1; i<N; i++) {
             double a,b,c;
//	     tip(x[i]); tip(y[i]);

             a= y[i] - p[0].y;
             b= p[0].x - x[i];
             c= x[i]*p[0].y - p[0].x*y[i];
//	     tip(a); tip(b); tip(c);
	     vx = y[i] - p[0].y; // a vector of the axis
	     vy = x[i] - p[0].x;
	     cos= (vx)/sqrt(vx*vx+vy*vy); // Get the cosine from the scalar product of <vx,vy> and <1,0>
	     sin= sqrt(1-cos*cos);
	     if (vy > 0) sin = -sin;	     
	     
             c= -a; a=b; b=c;
             c= - ( a*(x[i]+p[0].x)/2 + b*(y[i]+p[0].y)/2 );
             // the potential simetry line is a,b,c

//	     tip(a); tip(b); tip(c);
//	     tip(cx); tip(cy);
	     if ( abs(a*cx + b*cy + c) > EPS ) // This line doesn't pass trough the gravity mass, discard
	        continue;
	     
//	     cout << "Okay\n";
//	    tip(cos);	     
             double mat[9] = Ident;
//	     debug_chk_mat(mat);
             Rotate(mat, sin, cos); // Assuming that cos is the angle between the line and the Ox axis
//	     debug_chk_mat(mat);
             Translate(mat, 0, c/sqrt(a*a+b*b)); // I think the distance between the line and O is this
//	     debug_chk_mat(mat);
             Scale(mat, 1, -1); // Obtain the Symetrical
//	     debug_chk_mat(mat);
             Translate(mat, 0, -c/sqrt(a*a+b*b)); // Translate it Back!
//	     debug_chk_mat(mat);
             Rotate(mat, -sin, cos);
//	     debug_chk_mat(mat);
             // Now if you pass a point trough mat it should return it's symmetric
	     
             if ( check(mat) ) 
	     {
	        sol ++;             
		ax.push_back(a);
		bx.push_back(b);		
		cz.push_back(c);		
	     }
     }
//     cout << "Finnaly\n";
     // Finally check if the line that passes trough our point and the gravity center is an axis of symetry
     double a,b,c;
     a= cy - p[0].y;
     b= p[0].x - cx;
     c= cx*p[0].y - p[0].x*cy;
     
     // the potential simetry line is a,b,c
    vx = cx - p[0].x;
    vy = cy - p[0].y;
    cos = vx / sqrt( vx*vx + vy*vy);
    sin= sqrt(1-cos*cos);    
    if (vy < 0) sin = -sin;
//     if ( abs(a*cx + b*cy + c) > EPS ) // This line doesn't pass trough the gravity mass, discard
//        continue; Obsolete
		
     double mat[9] = Ident;
//     printf("%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.3lf\n\n", mat[0], mat[1], mat[2], mat[3], mat[4], mat[5], mat[6], mat[7], mat[8]);	          
     Rotate(mat, sin, cos); // Assuming that a/c is the sine of the angle between the line and the Ox axis
//     printf("%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.3lf\n\n", mat[0], mat[1], mat[2], mat[3], mat[4], mat[5], mat[6], mat[7], mat[8]);	     
     Translate(mat, 0, c/sqrt(a*a+b*b)); // I think the distance between the line and O is this
//     printf("%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.3lf\n\n", mat[0], mat[1], mat[2], mat[3], mat[4], mat[5], mat[6], mat[7], mat[8]);	     
     Scale(mat, 1, -1); // Obtain the Symetrical
//     printf("%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.3lf\n\n", mat[0], mat[1], mat[2], mat[3], mat[4], mat[5], mat[6], mat[7], mat[8]);	     
     Translate(mat, 0, -c/sqrt(a*a+b*b)); // Translate it Back!
//     printf("%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.3lf\n\n", mat[0], mat[1], mat[2], mat[3], mat[4], mat[5], mat[6], mat[7], mat[8]);	     
     Rotate(mat, -sin, cos);
//     printf("%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.2lf\n%3.2lf %3.2lf %3.3lf\n\n", mat[0], mat[1], mat[2], mat[3], mat[4], mat[5], mat[6], mat[7], mat[8]);	     
     // Now if you pass a point trough mat it should return it's symmetric

     if ( check(mat) )  {
        sol ++;          
	ax.push_back(a);
	bx.push_back(b);	
	cz.push_back(c);	
     }   
     
}


int main() {
    freopen(file".in", "rt", stdin);
    freopen(file".out", "wt", stdout);

    read_data();
    solve();
    
    printf("%d\n",sol);
    for (int i=0; i<sol; i++) 
    {
    printf ("%.8lf %.8lf %.8lf\n", ax[i], bx[i], cz[i]);
    }
    fclose(stdout);
    return 0;
}
