#include <assert.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

#define maxN 100000
#define maxM 100000
#define maxL 200000
#define INFI 0x3f3f3f3f

int Z[maxL], L;
int X[maxN], N;
int Y[maxM], M;

int pos[maxL];
char b_h[maxL];
int prm[maxL], cnt[maxL];
int bck[maxL], lcp[maxL];

int cmp_index(const void *a, const void *b)
{
    return Z[*(int *)a] - Z[*(int *)b];
}

/* suffix sort ala Emilian Miron */
void suffix_sort(void)
{
    int i, h, n_bck;
    
    /* sort after first number */
    for(i = 0; i < L; ++ i)
        pos[i] = i;
    qsort(pos, L, sizeof(int), cmp_index);

    /* facem o prima impartire in buchete */
    n_bck = 1;
    b_h[0] = 1;
    prm[pos[0]] = 0;

    for (i = 1; i < L; ++ i) {
        prm[pos[i]] = i;

        if(Z[pos[i]] != Z[pos[i - 1]])
            b_h[i] = 1, ++ n_bck;
        else b_h[i] = 0;
    }

    /* start sorting */
    for (h = 1; h < L; h *= 2) {
        int c_bck = INFI;

        /* punem fiecare suffix in buchetul sau */
        for(i = 0; i < L; ++ i) {
            if (b_h[i] != 0)
               cnt[c_bck = i] = 0;

            assert(c_bck != INFI);
            bck[pos[i]] = c_bck;
        }

        /* sortam */
        for(i = 0; i < L; ++ i) {
            const int d = pos[i] >= h ? pos[i] - h : pos[i] - h + L;
            const int e = bck[d];

            prm[d] = e + (cnt[e] ++);
        }

        for(i = 0; i < L; ++ i)
            pos[prm[i]] = i;

        for(i = 0; i < L; ++ i)
            if(b_h[i] == 0) {
                const int d = pos[i]   >= L - h ? pos[i]   + h - L : pos[i]   + h;
                const int e = pos[i-1] >= L - h ? pos[i-1] + h - L : pos[i-1] + h;

                if(bck[d] != bck[e])
                    b_h[i] = 1, ++ n_bck;
            }

        /* daca am terminat mai devreme */
        if(n_bck == L)
            break;
    }

    /* verificam */
    /*
    for(i = 1; i < L; ++ i) {
        int j;
        for(j = 0; j < L; ++ j) {
            const int b1 = Z[(pos[i]+j) % L];
            const int b2 = Z[(pos[i-1]+j) % L];
    
            assert(b1 >= b2);
            if(b1 > b2) break;
        }

        assert(pos[prm[i]] == i);
    }
    */
} 

/* longest common prefix */
void longest(void)
{
    int lung, unde;
    int i;
    
    lcp[L-1] = INFI;

    lung = 0, unde = pos[0];
    for(i = 0; i < L; ++ i) {
        int next = pos[prm[unde]+1];

        // fprintf(stderr, "[%d %d %d]\n", unde, next, lung);
        // assert(prm[unde] != L-1);
        if(lung < 0)
            lung = 0;
        while(Z[unde+lung] == Z[next+lung])
            ++ lung;

        lcp[prm[unde]] = lung;
        if(++ unde == L)
            unde = 0;
        lung --;
    }

    /* verificam */
    /*
    for(i = 0; i < L-1; ++ i) {
        int v1 = pos[i];
        int v2 = pos[i+1];
        int j;

        for(j = 0; j < lcp[i]; ++ j)
            assert(Z[v1+j] == Z[v2+j]);
        // assert(Z[v1+j] != Z[v2+j]);
    }
    */
}

void show(void)
{
    int i;

    fprintf(stderr, "Z = ");
    for(i = 0; i < L; ++ i)
        fprintf(stderr, "%4d ", Z[i]);
    fprintf(stderr, "\n");

    fprintf(stderr, "P = ");
    for(i = 0; i < L; ++ i)
        fprintf(stderr, "%4d ", pos[i]);
    fprintf(stderr, "\n");
    
    for(i = 0; i < L; ++ i) {
        int j;
        fprintf(stderr, "%4d / ", lcp[i]);
        for(j = 0; j < L; ++ j)
            fprintf(stderr, "%4d ", Z[(pos[i]+j) % L]);
        fprintf(stderr, "\n");
    }         

}

int main(void)
{
    int i;
    int best, bi, bj;
    
    freopen("siruri.in", "r", stdin);
    freopen("siruri.out", "w", stdout);
    
    scanf("%d", &N);
    for(i = 0; i < N; ++ i)
        scanf("%d", X+i);
        
    scanf("%d", &M);
    for(i = 0; i < M; ++ i)
        scanf("%d", Y+i);
	
    for(i = 0; i < N-1; ++ i)
        Z[i] = X[i] - X[i+1];
    Z[N-1] = INFI;
    
    for(i = 0; i < M-1; ++ i)
        Z[N+i] = Y[i+1] - Y[i];
    Z[N+M-1] = INFI+1;
    
    L = N+M;    

    /*    
    fprintf(stderr, "L = %d\n", L);
    for(i = 0; i < L; ++ i)
        fprintf(stderr, "% 3d", Z[i]);
    fprintf(stderr, "\n\n");
    */    

    suffix_sort();
    longest();
    // show(); 

    best = -INFI;
    bi = bj = -INFI;

    for(i = 0; i < L-1; ++ i) {
        if(lcp[i] <= best) continue;
        
        if(pos[i] < N && pos[i+1] >= N) {
            bi = pos[i];
            bj = pos[i+1];
            best = lcp[i];
            continue;
        }

        if(pos[i+1] < N && pos[i] >= N) {
            bi = pos[i+1];
            bj = pos[i];
            best = lcp[i];
            continue;
        }  
    }

    printf("%d %d %d\n", best+1, bi+1, bj-N+1);
    
    /*
    bj -= N;
    for(i = 0; i <= best; ++ i)
        assert(X[i+bi] + Y[i+bj] == X[bi] + Y[bj]); 
    */

    return 0;
}

