#include <stdio.h>
#include <assert.h>
#include <string.h>

#define maxL 800096
#define maxS 200025         /* including \0 */

typedef unsigned char uint8_t;
typedef struct huge huge;

struct huge {
    int lung, max;
    uint8_t *numb;
};

void create(huge *h, int lung)
{
    h->max = lung;
    h->numb = (uint8_t *)malloc(sizeof(uint8_t)*lung);
}

void set(huge *h, int scalar)
{
    memset(h->numb, 0x00, sizeof(uint8_t)*h->max);
    h->lung = 0;
    
    while(scalar) {
        h->numb[h->lung ++] = scalar & 1;
        scalar >>= 1;
    }
}

/* return h >= n*2^v */
int cmp(const huge *h, const huge *n, int v)
{
    int i;
    const int h_l = h->lung;
    const int n_l = n->lung;
    
    if(h_l > n_l + v)
        return 1;
    if(h_l < n_l + v)
        return 0;

    for(i = 0; i < h_l && i < n_l; ++ i) {
        if(h->numb[h_l-1-i] > n->numb[n_l-1-i])
            return 1;
        if(h->numb[h_l-1-i] < n->numb[n_l-1-i])
            return 0;
    }

    return 1;
}

/* scadere de doua numere mari a - b*2^v */
/* trebuie ca rezultatul sa fie mereu pozitiv */
void sub(huge *a, const huge *b, int v)
{
    int i, t = 0;

    for(i = 0; i < b->lung; ++ i) {
        t = a->numb[i+v] - b->numb[i] - t;
        
        if(t < 0) a->numb[i+v] = t+2, t = 1;
        else a->numb[i+v] = t, t = 0;
    }

    for(; i+v < a->lung && t; ++ i) {
        t = a->numb[i+v] - t;

        if(t < 0) a->numb[i+v] = t+2, t = 1;
        else a->numb[i+v] = t, t = 0;
    }

    assert(t == 0);

    while(a->numb[a->lung - 1] == 0 && a->lung > 0)
        -- a->lung;
}

void print(huge *a)
{
    int i;
    
    for(i = 0; i < a->lung; ++ i)
        fprintf(stderr, "%d", a->numb[i]);
    fprintf(stderr, "/%d\n", a->lung);
}

huge K, P, R;

int main(void)
{
    char s[maxL];
    int i, len;
    int N, T;
    
    freopen("supersf.in", "r", stdin);
    freopen("supersf.out", "w", stdout);

    fgets(s, maxS, stdin);
    len = strlen(s);
    if(s[len-1] == '\n')
        len --;

    create(&K, maxL);
    create(&R, maxL);
    set(&K, 0);
    set(&R, 0);

    for(i = len-1; i >= 0; -- i) {
        int dig;

        if('A' <= s[i])
            dig = s[i] - 'A' + 10;
        else
            dig = s[i] - '0';
        
        K.numb[(len-i)*4-4] = (dig & 0x01) != 0;
        K.numb[(len-i)*4-3] = (dig & 0x02) != 0;
        K.numb[(len-i)*4-2] = (dig & 0x04) != 0;
        K.numb[(len-i)*4-1] = (dig & 0x08) != 0;           
    }

    /* calculam lungimea lui K */   
    K.lung = len*4;
    while(K.numb[K.lung-1] == 0 && K.lung > 0)
        -- K.lung;

    /* scadem 1 */
    i = 0;
    while(K.numb[i] == 0)
        K.numb[i ++] = 1;
    K.numb[i] = 0;
    if(i == K.lung-1)
        K.lung --;

    if(K.lung == 0) {
        printf("1\n");
        return 0;
    }

    // fprintf(stderr, "K.lung = %d\n", K.lung);

    /* P contine pe N */
    create(&P, 32);
    
    T = N = 0;
    do {
        N ++;
        set(&P, N+T);
    } while(cmp(&K, &P, N-1) == 1);

    while(K.lung > 0 && N >= 0) {
        do {
            N --;
            if(N < 0)
                break;
            set(&P, N+2*T);
            // fprintf(stderr, ".%d %d\n", P.lung, N+T);
        } while(cmp(&K, &P, N-1) == 0);
        // fflush(stderr);
        
        if(N < 0)
            break;

        // fprintf(stderr, "N = %d %d %d\n", N, T, (N+2*T) << (N-1));
        // fprintf(stderr, "K = "); print(&K);
        // fprintf(stderr, "P = "); print(&P);

        if(R.lung == 0)
            R.lung = N+1;
        R.numb[N] = 1;

        sub(&K, &P, N-1);
        T ++;

        // fprintf(stderr, "K-P = "); print(&K);
    }

    for(i = (R.lung-1) | 3; i > 0; i -= 4) {
        int v = R.numb[i-0]*8 + R.numb[i-1]*4 + R.numb[i-2]*2 + R.numb[i-3];
        if(v < 10)
            printf("%c", (char)(v + '0'));
        else
            printf("%c", (char)(v-10 + 'A'));        
    }
    printf("\n");
    
    return 0;
}

